@@ -3,6 +3,7 @@ package stub
 import (
 	"fmt"
 	"os"
+	"runtime"
 	"strings"
 	"time"
 
@@ -54,10 +55,11 @@ func TestNoArchOrDist(t *testing.T) {
 	h, cfg, event := setup()
 	processCred(&h, cfg, t)
 	err := h.Handle(event)
-	statuses := []corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
+	// image in progress (4th entry, array index 3) should still be false when there is no content ... a la z or ppc
+	statuses := []corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
 	validate(true, err, "", cfg, conditions, statuses, t)
 	err = h.Handle(event)
-	statuses[0] = corev1.ConditionTrue
+	// and on a subsequent event, exists should still be false since we did not create any content previously
 	validate(true, err, "", cfg, conditions, statuses, t)
 }
 
@@ -65,20 +67,33 @@ func TestWithDist(t *testing.T) {
 	h, cfg, event := setup()
 	processCred(&h, cfg, t)
 	err := h.Handle(event)
-	statuses := []corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
+	statuses := []corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
 	validate(true, err, "", cfg, conditions, statuses, t)
 	err = h.Handle(event)
-	statuses = []corev1.ConditionStatus{corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
+	// image in progress (4th entry, array index 3) should still be false when there is no content ... a la z or ppc
+	statuses = []corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
+	validate(true, err, "", cfg, conditions, statuses, t)
+	mimic(&h, x86OCPContentRootDir)
+	err = h.Handle(event)
+	// with content present, image im progress should now be true
+	statuses = []corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
 	validate(true, err, "", cfg, conditions, statuses, t)
-
 }
 
 func TestWithArchDist(t *testing.T) {
 	h, cfg, event := setup()
 	processCred(&h, cfg, t)
-	cfg.Spec.Architectures = []string{
-		v1.X86Architecture,
+	if len(cfg.Spec.Architectures) == 0 {
+		t.Errorf("arch not set on bootstrap")
 	}
+	testValue := runtime.GOARCH
+	if testValue == v1.AMDArchitecture {
+		testValue = v1.X86Architecture
+	}
+	if cfg.Spec.Architectures[0] != testValue {
+		t.Errorf("arch set to %s instead of %s", cfg.Spec.Architectures[0], runtime.GOARCH)
+	}
+
 	mimic(&h, x86OCPContentRootDir)
 	err := h.Handle(event)
 	statuses := []corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
@@ -91,16 +106,18 @@ func TestWithArchDist(t *testing.T) {
 		conditions,
 		statuses, t)
 
+
 }
 
 func TestWithArch(t *testing.T) {
 	h, cfg, event := setup()
 	processCred(&h, cfg, t)
+	// without a mimic call this simulates our current PPC/390 stories of no samples content
 	cfg.Spec.Architectures = []string{
-		v1.X86Architecture,
+		v1.PPCArchitecture,
 	}
 	err := h.Handle(event)
-	validate(true, err, "", cfg, conditions, []corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}, t)
+	validateArchOverride(true, err, "", cfg, conditions, []corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}, t, v1.PPCArchitecture)
 }
 
 func TestWithBadArch(t *testing.T) {
@@ -221,7 +238,7 @@ func TestSkipped(t *testing.T) {
 	mimic(&h, x86OCPContentRootDir)
 
 	err := h.Handle(event)
-	validate(true, err, "", cfg, conditions, []corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}, t)
+	validate(true, err, "", cfg, conditions, []corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}, t)
 
 	fakeisclient := h.imageclientwrapper.(*fakeImageStreamClientWrapper)
 	for _, key := range iskeys {
@@ -474,7 +491,14 @@ func TestImageStreamErrorRetry(t *testing.T) {
 						Name: "foofoo",
 					},
 				},
-			},
+				{
+					Name:       "bar",
+					Generation: &tagVersion,
+					From: &corev1.ObjectReference{
+						Kind: "DockerImage",
+						Name: "foofoo",
+					},
+				}},
 		},
 		Status: imagev1.ImageStreamStatus{
 			Tags: []imagev1.NamedTagEventList{
@@ -483,12 +507,21 @@ func TestImageStreamErrorRetry(t *testing.T) {
 					Conditions: []imagev1.TagEventCondition{
 						{
 							Generation: tagVersion,
-							Status: corev1.ConditionFalse,
-							Message: "failed import",
+							Status:     corev1.ConditionFalse,
+							Message:    "failed import",
 						},
 					},
 				},
-			},
+				{
+					Tag: "bar",
+					Conditions: []imagev1.TagEventCondition{
+						{
+							Generation: tagVersion,
+							Status:     corev1.ConditionFalse,
+							Message:    "failed import",
+						},
+					},
+				}},
 		},
 	}
 
@@ -498,6 +531,13 @@ func TestImageStreamErrorRetry(t *testing.T) {
 		t.Fatalf("Import Error Condition not true: %#v", cfg)
 	}
 
+	fakeisclient := h.imageclientwrapper.(*fakeImageStreamClientWrapper)
+	fakeimporter := fakeisclient.ImageStreamImports("foo").(*fakeImageStreamImporter)
+
+	if fakeimporter.count != 2 {
+		t.Fatalf("incorrect amount of import calls %d", fakeimporter.count)
+	}
+
 	initialImportErrorLastUpdateTime := cfg.Condition(v1.ImportImageErrorsExist).LastUpdateTime
 	h.processImageStreamWatchEvent(is, false)
 	// refetch to see if updated
@@ -507,26 +547,32 @@ func TestImageStreamErrorRetry(t *testing.T) {
 	}
 
 	// now let's push back 15 minutes and update
-	importError.LastUpdateTime.Time = metav1.Now().Add(-15 * time.Minute)
+	lastUpdateTime, _ := h.imagestreamRetry[is.Name]
+	lastUpdateTime.Time = metav1.Now().Add(-15 * time.Minute)
+	h.imagestreamRetry[is.Name] = lastUpdateTime
 	cfg.ConditionUpdate(importError)
 	// save a copy for compare
-	fifteenMinutesAgo := importError.LastUpdateTime
+	fifteenMinutesAgo := lastUpdateTime
 
 	h.processImageStreamWatchEvent(is, false)
 
 	// refetch and make sure it has changed
-	if cfg.Condition(v1.ImportImageErrorsExist).LastUpdateTime.Equal(&fifteenMinutesAgo) {
+	lastUpdateTime, _ = h.imagestreamRetry[is.Name]
+	if lastUpdateTime.Equal(&fifteenMinutesAgo) {
 		t.Fatalf("Import Error Condition should have been updated: old update time %s new update time %s", initialImportErrorLastUpdateTime.String(), cfg.Condition(v1.ImportImageErrorsExist).LastUpdateTime.String())
 	}
 
 	tagVersion = int64(2)
-	is.Status.Tags[0].Conditions[0] = imagev1.TagEventCondition{}
-	is.Status.Tags[0].Items = []imagev1.TagEvent{
-		{
-			Generation: tagVersion,
-		},
+	for _, tag := range is.Status.Tags {
+		tag.Conditions[0] = imagev1.TagEventCondition{}
+		tag.Items = []imagev1.TagEvent{
+			{
+				Generation: tagVersion,
+			},
+		}
 	}
 
+
 	h.processImageStreamWatchEvent(is, false)
 
 	if cfg.ConditionTrue(v1.ImportImageErrorsExist) {
@@ -575,6 +621,7 @@ func TestCreateDeleteSecretBeforeCR(t *testing.T) {
 			ResourceVersion: "a",
 		},
 	}
+	mimic(&h, x86OCPContentRootDir)
 
 	err := h.Handle(event)
 	validate(false, err, "Received secret samples-registry-credentials but do not have the Config yet", cfg,
@@ -607,7 +654,7 @@ func TestCreateDeleteSecretBeforeCR(t *testing.T) {
 
 func TestCreateDeleteSecretAfterCR(t *testing.T) {
 	h, cfg, event := setup()
-
+	mimic(&h, x86OCPContentRootDir)
 	err := h.Handle(event)
 	statuses := []corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
 	validate(true, err, "", cfg, conditions, statuses, t)
@@ -678,7 +725,7 @@ func processCred(h *Handler, cfg *v1.Config, t *testing.T) {
 
 func TestSameSecret(t *testing.T) {
 	h, cfg, event := setup()
-
+	mimic(&h, x86OCPContentRootDir)
 	err := h.Handle(event)
 	statuses := []corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
 	validate(true, err, "", cfg, conditions, statuses, t)
@@ -1071,20 +1118,22 @@ func TestDeletedCR(t *testing.T) {
 
 func TestSameCR(t *testing.T) {
 	h, cfg, event := setup()
+	mimic(&h, x86OCPContentRootDir)
 	processCred(&h, cfg, t)
 	cfg.ResourceVersion = "a"
 
-	// first pass on the resource creates the samples
+	// first pass on the resource creates the samples, exists (first entry, index 0) is still false
 	statuses := []corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
 	err := h.Handle(event)
 	validate(true, err, "", cfg, conditions, statuses, t)
 
 	err = h.Handle(event)
+	// next pass is when we expect exists to be true
 	statuses[0] = corev1.ConditionTrue
 	validate(true, err, "", cfg, conditions, statuses, t)
 
 	err = h.Handle(event)
-	statuses[3] = corev1.ConditionFalse
+	// now with content, should see no change in status after duplicate event, where imagestream import status has not changed
 	validate(true, err, "", cfg, conditions, statuses, t)
 
 }
@@ -1112,6 +1161,7 @@ func TestBadSubDirList(t *testing.T) {
 
 func TestBadTopLevelStatus(t *testing.T) {
 	h, cfg, event := setup()
+	mimic(&h, x86OCPContentRootDir)
 	processCred(&h, cfg, t)
 	fakestatus := h.crdwrapper.(*fakeCRDWrapper)
 	fakestatus.updateerr = fmt.Errorf("badsdkupdate")
@@ -1239,7 +1289,7 @@ func mimic(h *Handler, topdir string) {
 
 }
 
-func validate(succeed bool, err error, errstr string, cfg *v1.Config, statuses []v1.ConfigConditionType, conditions []corev1.ConditionStatus, t *testing.T) {
+func validateArchOverride(succeed bool, err error, errstr string, cfg *v1.Config, statuses []v1.ConfigConditionType, conditions []corev1.ConditionStatus, t *testing.T, arch string) {
 	if succeed && err != nil {
 		t.Fatal(err)
 	}
@@ -1266,9 +1316,22 @@ func validate(succeed bool, err error, errstr string, cfg *v1.Config, statuses [
 				t.Fatalf("unexpected for succeed %v have status condition %#v expected condition %#v and status %#v", succeed, cfg.Status.Conditions[i], c, conditions[i])
 			}
 		}
+		testValue := arch
+		if testValue == v1.AMDArchitecture {
+			testValue = v1.X86Architecture
+		}
+		if cfg.Spec.Architectures[0] != testValue {
+			t.Fatalf("arch set to %s instead of %s", cfg.Spec.Architectures[0], runtime.GOARCH)
+		}
 	}
+
 }
 
+func validate(succeed bool, err error, errstr string, cfg *v1.Config, statuses []v1.ConfigConditionType, conditions []corev1.ConditionStatus, t *testing.T) {
+	validateArchOverride(succeed, err, errstr, cfg, statuses, conditions, t, runtime.GOARCH)
+}
+
+
 func NewTestHandler() Handler {
 	h := Handler{}
 
@@ -1318,6 +1381,7 @@ func NewTestHandler() Handler {
 
 	h.imagestreamFile = make(map[string]string)
 	h.templateFile = make(map[string]string)
+	h.imagestreamRetry = make(map[string]metav1.Time)
 	h.version = TestVersion
 
 	return h
@@ -1408,11 +1472,12 @@ func (f *fakeResourceFileLister) List(dir string) ([]os.FileInfo, error) {
 }
 
 type fakeImageStreamClientWrapper struct {
-	streams      map[string]*imagev1.ImageStream
-	upsertkeys   map[string]bool
-	geterrors    map[string]error
-	upserterrors map[string]error
-	listerrors   map[string]error
+	streams                 map[string]*imagev1.ImageStream
+	upsertkeys              map[string]bool
+	geterrors               map[string]error
+	upserterrors            map[string]error
+	listerrors              map[string]error
+	fakeImageStreamImporter *fakeImageStreamImporter
 }
 
 func (f *fakeImageStreamClientWrapper) Get(name string) (*imagev1.ImageStream, error) {
@@ -1462,12 +1527,18 @@ func (f *fakeImageStreamClientWrapper) Watch() (watch.Interface, error) {
 }
 
 func (f *fakeImageStreamClientWrapper) ImageStreamImports(namespace string) imagev1client.ImageStreamImportInterface {
-	return &fakeImageStreamImporter{}
+	if f.fakeImageStreamImporter == nil {
+		f.fakeImageStreamImporter = &fakeImageStreamImporter{}
+	}
+	return f.fakeImageStreamImporter
 }
 
-type fakeImageStreamImporter struct {}
+type fakeImageStreamImporter struct {
+	count int
+}
 
 func (f *fakeImageStreamImporter) Create(*imagev1.ImageStreamImport) (*imagev1.ImageStreamImport, error) {
+	f.count++
 	return &imagev1.ImageStreamImport{}, nil
 }
 
